@@ -198,13 +198,13 @@ int socket_set_reusable(evutil_socket_t fd, int flag, SOCKET_TYPE st) {
 #if defined(WINDOWS)
     int use_reuseaddr = IS_TURN_SERVER;
 #else
-    int use_reuseaddr = 1;
+    const int use_reuseaddr = 1;
 #endif
 
 #if defined(SO_REUSEADDR)
     if (use_reuseaddr) {
       int on = flag;
-      int ret = setsockopt(fd, SOL_SOCKET, SO_REUSEADDR, (const void *)&on, (socklen_t)sizeof(on));
+      const int ret = setsockopt(fd, SOL_SOCKET, SO_REUSEADDR, (const void *)&on, (socklen_t)sizeof(on));
       if (ret < 0) {
         perror("SO_REUSEADDR");
       }
@@ -318,10 +318,10 @@ int addr_bind(evutil_socket_t fd, const ioa_addr *addr, int reusable, int debug,
     }
     if (ret < 0) {
       if (debug) {
-        int err = socket_errno();
+        const int err = socket_errno();
         perror("bind");
-        char str[129];
-        addr_to_string(addr, (uint8_t *)str);
+        char str[MAX_IOA_ADDR_STRING];
+        addr_to_string(addr, str);
         TURN_LOG_FUNC(TURN_LOG_LEVEL_WARNING, "Trying to bind fd %d to <%s>: errno=%d\n", fd, str, err);
       }
     }
@@ -670,7 +670,7 @@ int set_mtu_df(SSL *ssl, evutil_socket_t fd, int family, int mtu, int df_value,
     return 0;
   }
 
-  int ret = set_socket_df(fd, family, df_value);
+  const int ret = set_socket_df(fd, family, df_value);
 
   if (!mtu) {
     mtu = SOSO_MTU;
@@ -919,12 +919,19 @@ static char *_WTA(__in wchar_t *pszInBuf, __in int nInSize, __out char **pszOutB
   // and we have to add space to the allocation ourselves.
   (*pnOutSize)++;
   *pszOutBuf = malloc(*pnOutSize * sizeof(char));
+  if (!pszOutBuf) {
+    return NULL;
+  }
   if (WideCharToMultiByte((UINT)0, (DWORD)0, pszInBuf, nInSize, *pszOutBuf, *pnOutSize, NULL, NULL) == 0) {
     free(*pszOutBuf);
     return NULL;
   } else {
-    (*pszOutBuf)[*pnOutSize - 1] = '\0';
-    return *pszOutBuf;
+    if (pszOutBuf != NULL) {
+      (*pszOutBuf)[*pnOutSize - 1] = '\0';
+      return *pszOutBuf;
+    } else {
+      return NULL;
+    }
   }
 }
 
@@ -1094,7 +1101,7 @@ void print_abs_file_name(const char *msg1, const char *msg2, const char *fn) {
         if (!getcwd(absfn, sizeof(absfn) - 1)) {
           absfn[0] = 0;
         }
-        size_t blen = strlen(absfn);
+        const size_t blen = strlen(absfn);
         if (blen < sizeof(absfn) - 1) {
           strncpy(absfn + blen, "/", sizeof(absfn) - blen);
           strncpy(absfn + blen + 1, fn, sizeof(absfn) - blen - 1);
@@ -1122,10 +1129,10 @@ char *find_config_file(const char *config_file) {
       }
     } else {
       int i = 0;
-      size_t cflen = strlen(config_file);
+      const size_t cflen = strlen(config_file);
 
       while (config_file_search_dirs[i]) {
-        size_t dirlen = strlen(config_file_search_dirs[i]);
+        const size_t dirlen = strlen(config_file_search_dirs[i]);
         size_t fnsz = sizeof(char) * (dirlen + cflen + 10);
         char *fn = (char *)malloc(fnsz + 1);
         strncpy(fn, config_file_search_dirs[i], fnsz);
@@ -1139,7 +1146,7 @@ char *find_config_file(const char *config_file) {
         }
         free(fn);
         if (config_file_search_dirs[i][0] != '/' && config_file_search_dirs[i][0] != '.' && c_execdir && c_execdir[0]) {
-          size_t celen = strlen(c_execdir);
+          const size_t celen = strlen(c_execdir);
           fnsz = sizeof(char) * (dirlen + cflen + celen + 10);
           fn = (char *)malloc(fnsz + 1);
           strncpy(fn, c_execdir, fnsz);
@@ -1198,8 +1205,8 @@ static uint64_t turn_getRandTime(void) {
 #else
   tp.tv_sec = time(NULL);
 #endif
-  uint64_t current_time = (uint64_t)(tp.tv_sec);
-  uint64_t current_mstime = (uint64_t)(current_time + (tp.tv_nsec));
+  const uint64_t current_time = (uint64_t)(tp.tv_sec);
+  const uint64_t current_mstime = (uint64_t)(current_time + (tp.tv_nsec));
 
   return current_mstime;
 }
@@ -1212,14 +1219,6 @@ void turn_srandom(void) {
 #endif
 }
 
-long turn_random(void) {
-#if defined(WINDOWS)
-  return rand();
-#else
-  return random();
-#endif
-}
-
 unsigned long set_system_parameters(int max_resources) {
   turn_srandom();
 
@@ -1311,11 +1310,11 @@ char *base64_encode(const unsigned char *data, size_t input_length, size_t *outp
 
   for (size_t i = 0, j = 0; i < input_length;) {
 
-    uint32_t octet_a = i < input_length ? data[i++] : 0;
-    uint32_t octet_b = i < input_length ? data[i++] : 0;
-    uint32_t octet_c = i < input_length ? data[i++] : 0;
+    const uint32_t octet_a = i < input_length ? data[i++] : 0;
+    const uint32_t octet_b = i < input_length ? data[i++] : 0;
+    const uint32_t octet_c = i < input_length ? data[i++] : 0;
 
-    uint32_t triple = (octet_a << 0x10) + (octet_b << 0x08) + octet_c;
+    const uint32_t triple = (octet_a << 0x10) + (octet_b << 0x08) + octet_c;
 
     encoded_data[j++] = encoding_table[(triple >> 3 * 6) & 0x3F];
     encoded_data[j++] = encoding_table[(triple >> 2 * 6) & 0x3F];
@@ -1336,10 +1335,12 @@ void build_base64_decoding_table(void) {
 
   char *table = (char *)calloc(256, sizeof(char));
 
-  for (size_t i = 0; i < 64; i++) {
-    table[(unsigned char)encoding_table[i]] = i;
+  if (table) {
+    for (size_t i = 0; i < 64; i++) {
+      table[(unsigned char)encoding_table[i]] = i;
+    }
+    decoding_table = table;
   }
-  decoding_table = table;
 }
 
 unsigned char *base64_decode(const char *data, size_t input_length, size_t *output_length) {
@@ -1369,12 +1370,12 @@ unsigned char *base64_decode(const char *data, size_t input_length, size_t *outp
   size_t j;
   for (i = 0, j = 0; i < (int)input_length;) {
 
-    uint32_t sextet_a = data[i] == '=' ? 0 & i++ : decoding_table[(int)data[i++]];
-    uint32_t sextet_b = data[i] == '=' ? 0 & i++ : decoding_table[(int)data[i++]];
-    uint32_t sextet_c = data[i] == '=' ? 0 & i++ : decoding_table[(int)data[i++]];
-    uint32_t sextet_d = data[i] == '=' ? 0 & i++ : decoding_table[(int)data[i++]];
+    const uint32_t sextet_a = data[i] == '=' ? 0 & i++ : decoding_table[(int)data[i++]];
+    const uint32_t sextet_b = data[i] == '=' ? 0 & i++ : decoding_table[(int)data[i++]];
+    const uint32_t sextet_c = data[i] == '=' ? 0 & i++ : decoding_table[(int)data[i++]];
+    const uint32_t sextet_d = data[i] == '=' ? 0 & i++ : decoding_table[(int)data[i++]];
 
-    uint32_t triple = (sextet_a << 3 * 6) + (sextet_b << 2 * 6) + (sextet_c << 1 * 6) + (sextet_d << 0 * 6);
+    const uint32_t triple = (sextet_a << 3 * 6) + (sextet_b << 2 * 6) + (sextet_c << 1 * 6) + (sextet_d << 0 * 6);
 
     if (j < *output_length) {
       decoded_data[j++] = (triple >> 2 * 8) & 0xFF;
@@ -1407,10 +1408,10 @@ const char *turn_get_ssl_method(SSL *ssl, const char *mdefault) {
 
 struct event_base *turn_event_base_new(void) {
   struct event_config *cfg = event_config_new();
-
   event_config_set_flag(cfg, EVENT_BASE_FLAG_EPOLL_USE_CHANGELIST);
-
-  return event_base_new_with_config(cfg);
+  struct event_base *base = event_base_new_with_config(cfg);
+  event_config_free(cfg); // Free the config after use to make valgrind happy
+  return base;
 }
 
 /////////// OAUTH /////////////////
