@@ -68,16 +68,20 @@
 static volatile int _log_file_line_set = 0;
 
 static volatile turn_time_t log_start_time = 0;
-volatile int _log_time_value_set = 0;
-volatile turn_time_t _log_time_value = 0;
+#if defined(WINDOWS)
+volatile uint32_t _log_time_value = 0;
+#else
+_Atomic uint32_t _log_time_value = 0;
+#endif
 
 static inline turn_time_t log_time(void) {
   if (!log_start_time) {
     log_start_time = turn_time();
   }
 
-  if (_log_time_value_set) {
-    return (_log_time_value - log_start_time);
+  const turn_time_t t = LOAD_LOG_TIME();
+  if (t) {
+    return (t - log_start_time);
   }
 
   return (turn_time() - log_start_time);
@@ -216,7 +220,7 @@ void set_syslog_facility(char *val) {
     return;
   }
 #if defined(__unix__) || defined(unix) || defined(__APPLE__)
-  int tmp = str_to_syslog_facility(val);
+  const int tmp = str_to_syslog_facility(val);
   if (tmp == -1) {
     TURN_LOG_FUNC(TURN_LOG_LEVEL_WARNING, "WARNING: invalid syslog-facility value (%s); ignored.\n", val);
     return;
@@ -578,7 +582,7 @@ void turn_log_func_default(const char *const file, const int line, const TURN_LO
   char s[MAX_RTPPRINTF_BUFFER_SIZE + 1];
   size_t so_far = 0;
   if (use_new_log_timestamp_format) {
-    time_t now = time(NULL);
+    const time_t now = time(NULL);
     so_far += strftime(s, sizeof(s), turn_log_timestamp_format, localtime(&now));
   } else {
     so_far += snprintf(s, sizeof(s), "%lu: ", (unsigned long)log_time());
@@ -721,7 +725,7 @@ int get_canonic_origin(const char *o, char *co, int sz) {
     if (uri) {
       const char *scheme = evhttp_uri_get_scheme(uri);
       if (scheme && scheme[0]) {
-        size_t schlen = strlen(scheme);
+        const size_t schlen = strlen(scheme);
         if ((schlen < (size_t)sz) && (schlen < STUN_MAX_ORIGIN_SIZE)) {
           const char *host = evhttp_uri_get_host(uri);
           if (host && host[0]) {
