@@ -43,6 +43,9 @@
 
 #if !defined(TURN_NO_PROMETHEUS)
 
+prom_counter_t *packet_processed;
+prom_counter_t *packet_dropped;
+
 prom_counter_t *stun_binding_request;
 prom_counter_t *stun_binding_response;
 prom_counter_t *stun_binding_error;
@@ -112,7 +115,7 @@ MHD_RESULT promhttp_handler(void *cls, struct MHD_Connection *connection, const
 }
 
 void start_prometheus_server(void) {
-  if (turn_params.prometheus == 0) {
+  if (!turn_params.prometheus) {
     TURN_LOG_FUNC(TURN_LOG_LEVEL_INFO, "prometheus collector disabled, not started\n");
     return;
   }
@@ -178,6 +181,11 @@ void start_prometheus_server(void) {
   const char *typeLabel[] = {"type"};
   turn_total_allocations = prom_collector_registry_must_register_metric(
       prom_gauge_new("turn_total_allocations", "Represents current allocations number", 1, typeLabel));
+  // Packet counters
+  packet_processed = prom_collector_registry_must_register_metric(
+      prom_counter_new("turn_packet_processed", "Incoming packet processed", 0, NULL));
+  packet_dropped = prom_collector_registry_must_register_metric(
+      prom_counter_new("turn_packet_dropped", "Incoming packet dropped", 0, NULL));
 
   // some flags appeared first in microhttpd v0.9.53
   unsigned int flags = 0;
@@ -221,7 +229,7 @@ void start_prometheus_server(void) {
     }
   }
 
-  uint8_t addr[MAX_IOA_ADDR_STRING];
+  char addr[MAX_IOA_ADDR_STRING] = "";
   addr_to_string(&server_addr, addr);
   TURN_LOG_FUNC(TURN_LOG_LEVEL_INFO, "prometheus exporter server will listen on %s\n", addr);
 
@@ -238,7 +246,7 @@ void start_prometheus_server(void) {
 
 void prom_set_finished_traffic(const char *realm, const char *user, unsigned long rsvp, unsigned long rsvb,
                                unsigned long sentp, unsigned long sentb, bool peer) {
-  if (turn_params.prometheus == 1) {
+  if (turn_params.prometheus) {
 
     const char *label[] = {realm, NULL};
     if (turn_params.prometheus_username_labels) {
@@ -270,33 +278,45 @@ void prom_set_finished_traffic(const char *realm, const char *user, unsigned lon
 }
 
 void prom_inc_allocation(SOCKET_TYPE type) {
-  if (turn_params.prometheus == 1) {
+  if (turn_params.prometheus) {
     const char *label[] = {socket_type_name(type)};
     prom_gauge_inc(turn_total_allocations, label);
   }
 }
 
 void prom_dec_allocation(SOCKET_TYPE type) {
-  if (turn_params.prometheus == 1) {
+  if (turn_params.prometheus) {
     const char *label[] = {socket_type_name(type)};
     prom_gauge_dec(turn_total_allocations, label);
   }
 }
 
+void prom_inc_packet_processed(int count) {
+  if (turn_params.prometheus) {
+    prom_counter_add(packet_processed, count, NULL);
+  }
+}
+
+void prom_inc_packet_dropped(int count) {
+  if (turn_params.prometheus) {
+    prom_counter_add(packet_dropped, count, NULL);
+  }
+}
+
 void prom_inc_stun_binding_request(void) {
-  if (turn_params.prometheus == 1) {
+  if (turn_params.prometheus) {
     prom_counter_add(stun_binding_request, 1, NULL);
   }
 }
 
 void prom_inc_stun_binding_response(void) {
-  if (turn_params.prometheus == 1) {
+  if (turn_params.prometheus) {
     prom_counter_add(stun_binding_response, 1, NULL);
   }
 }
 
 void prom_inc_stun_binding_error(void) {
-  if (turn_params.prometheus == 1) {
+  if (turn_params.prometheus) {
     prom_counter_add(stun_binding_error, 1, NULL);
   }
 }
@@ -339,4 +359,8 @@ void prom_inc_allocation(SOCKET_TYPE type) { UNUSED_ARG(type); }
 
 void prom_dec_allocation(SOCKET_TYPE type) { UNUSED_ARG(type); }
 
+void prom_inc_packet_processed(int count) { UNUSED_ARG(count); }
+
+void prom_inc_packet_dropped(int count) { UNUSED_ARG(count); }
+
 #endif /* TURN_NO_PROMETHEUS */
